"""
This script contains several functions built around matplotlib library, producing plots useful for initial data exploration.

Written by Roman Kastusik at The Data Analysis Bureau
"""
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd


def feature_to_title(name):
    title = ''
    for word in name.split('_'):
        title = title + ' ' + word
    return title


def categorical_plot(data, feature, figsize = (22, 6), font = 16, relative = False, log = False):
    """
    function produces a bar plot of categorical data

    Args:
    -------------
        - data [pandas dataframe]
        - feature [str] - name of one of the columns in data
        - figsize  [tuple (x,y)] - (optional, default: (22,6)) size of the figure to be produced
        - font  [int] - (optional, default: 16) fontsize to be used throughtout the plot for ticks, etc.
        - relative [boolean] - (optional, default: False) if True, shows relative frequency of the value, if False - absolute number of rows
        - log [boolean] - (optional, default: False), if True, y axis is log-scale

    Returns:
    -------------
    None, prints an image
    """
    if relative:
        norm = data.shape[0]
        y_label = 'Frequency'
    else:
        norm = 1
        y_label = 'No of rows'

    cc = []
    labels = []
    c = 1

    plt.figure(figsize = figsize)
    plt.title(feature_to_title(feature))
    plt.grid(zorder = 0)
    plt.ylabel(y_label, fontsize = font)
    if log:
        plt.yscale('symlog')

    for value in data[feature].unique():
        plt.bar(c, height = data[data[feature] == value].shape[0] / norm, alpha = 1, zorder = 2)
        labels.append(value)
        cc.append(c)
        c += 1

    plt.xticks(cc, labels = labels, fontsize = font)
    plt.yticks(fontsize = font)
    plt.show()


def distribution_plot(data, feature, xlog = False, ylog = False, font = 16, n_bins = 20):


    plt.figure(figsize = (22, 6))
    plt.grid(zorder = 0)
    plt.set_cmap('inferno')
    plt.xlabel(feature_to_title(feature), fontsize = font)
    plt.xticks(fontsize = 16)
    plt.ylabel('Samples', fontsize = font)
    plt.yticks(fontsize = font)
    if xlog:
        plt.xscale('symlog')
    if ylog:
        plt.yscale('symlog')
    n, bins, patches = plt.hist(data[feature],\
                                density = False,\
                                histtype = 'stepfilled',\
                                align = 'mid', \
                                alpha = 0.8,\
                               zorder = 2,\
                               bins = n_bins)

    plt.show()


def relationship_scatter(data_, feature_x, feature_y, filter_by = None, filter_more_than = None,\
                         filter_less_than = None, ylog = False, xlog = False, marker_size =10,\
                         tick_font = 14, label_font = 14, alpha = 0.4, polyfit = False, n_fit = 1):
    data = data_
    if filter_by:
        if filter_more_than == None and filter_less_than == None:
            raise ValueError('Limits of filter not provided')
        elif filter_more_than == None and filter_less_than != None:
            data_ = data_[data_[filter_by]<filter_less_than]
        elif filter_more_than != None and filter_less_than == None:
            data_ = data_[data_[filter_by]>filter_more_than]
        elif filter_more_than != None and filter_less_than != None:
            mask1 = data_[filter_by] > filter_more_than
            mask2 = data_[filter_by] < filter_less_than
            data_ = data_[mask1 & mask2]
    if data_.shape[0] < data.shape[0] * 0.5:
        alpha = alpha + 0.25

    plt.figure(figsize = (22, 6))
    plt.grid(zorder = 0)
    if filter_by:
        plt.title('Filtered by {}<{}<{}'.format(filter_more_than, filter_by, filter_less_than))
    plt.xlabel(feature_x, fontsize = label_font)
    plt.ylabel(feature_y, fontsize = label_font)
    if ylog:
        plt.yscale('symlog')
    if xlog:
        plt.xscale('symlog')
    plt.xticks(fontsize = tick_font)
    plt.yticks(fontsize = tick_font)

    plt.scatter(data_[feature_x], data_[feature_y], s = marker_size, alpha = alpha, zorder = 1)


    #            this whole polyfit thig needs a re-work      #######

    if polyfit:
        coefs = np.polyfit(data_[feature_x], data_[feature_y], n_fit)
        x = np.arange(data_[feature_x].min(), data_[feature_x].max())
        x = np.array([data_[feature_x].min(), data_[feature_x].max()])
        y = np.zeros(x.shape[0])
#         print(coefs)
#         print('x', x)
#         print('y',y)
#         print(x[1]*coefs[0] + coefs[1])
#         y_1 = coefs[0] * (np.log(x)/np.log(10)) + coefs[1]
# #         y = 10**y_1
        y = coefs[0]*x + coefs[1]
        plt.plot(x, y, '--k', linewidth = 1.5 )


    plt.show()

if __name__ == '__main__':
    pass 
